;==============================================================================
.code
;==============================================================================

;--------------------------------------------------
; Equates for the FPU status word exception flags.
;--------------------------------------------------

FEX_INVALID      equ 1
FEX_DENORMALIZED equ 2
FEX_ZERODIVIDE   equ 4
FEX_OVERFLOW     equ 8
FEX_UNDERFLOW    equ 16
FEX_PRECISION    equ 32
FEX_STACKFAULT   equ 64
FEX_ANY          equ 127

;------------------------------------------------------------
; This macro function returns the FPU status word ANDed with
; the specified exception mask (or masks combined with OR).
;------------------------------------------------------------

FpuStatus MACRO xmask
    IFNDEF _fpu_status_word_
        .data
            _fpu_status_word_ dw 0
        .code
    ENDIF
    fstsw _fpu_status_word_
    mov eax, xmask
    and eax, _fpu_status_word_
    EXITM <eax>
ENDM

;---------------------------------------------------------------------
; If an exception condition causes an exception status bit in the FPU
; status word to be set, it will remain set until it is explicitly
; cleared. This macro clears all of the exception status bits.
;---------------------------------------------------------------------

FCLEAREX MACRO
    fclex
ENDM

;==============================================================================

FIM_INVALID      equ 1
FIM_DENORMALIZED equ 2
FIM_ZERODIVIDE   equ 4
FIM_OVERFLOW     equ 8
FIM_UNDERFLOW    equ 16
FIM_PRECISION    equ 32

;---------------------------------------------------------------------
; These macros set or clear the specified interrupt mask bit (or mask
; bits combined with OR) in the FPU Control Word.
;
; In the initialized state all of the interrupt mask bits are set,
; effectively disabling the FPU interrupt and preventing the FPU
; from triggering external exceptions. Clearing a mask bit enables
; the FPU interrupt for the specified exception.
;---------------------------------------------------------------------

FSETIM MACRO imask
    IFNDEF _fpu_control_word_
        .data
            _fpu_control_word_ dw 0
        .code
    ENDIF
    fstcw _fpu_control_word_
    or  _fpu_control_word_, imask
    fldcw _fpu_control_word_
ENDM

FCLEARIM MACRO imask
    IFNDEF _fpu_control_word_
        .data
            _fpu_control_word_ dw 0
        .code
    ENDIF
    fstcw _fpu_control_word_
    and  _fpu_control_word_, NOT imask
    fldcw _fpu_control_word_
ENDM
