;==============================================================================

LPEXCEPTION_POINTERS typedef ptr EXCEPTION_POINTERS

;-----------------------------------------------------------------
; This macro sets the top-level exception filter for the calling
; process to our try_filter procedure, so when an unhandled
; exception reaches the UnhandledExceptionFilter function, and
; the process is not being debugged, the function will call our
; procedure. The return_address parameter specifies the address
; of a label were execution will continue if an exception occurs.
; Use this macro to begin a guarded section of code.
;-----------------------------------------------------------------

try_begin macro return_address
    push return_address
    pop _exception_return_address_
    invoke SetUnhandledExceptionFilter, try_filter
    mov _previous_exception_filter_, eax
endm

;----------------------------------------------------------------
; This macro sets the top-level exception filter for the calling
; process to what it was before the try_begin macro changed it.
; Use this macro to end a guarded section of code.
;----------------------------------------------------------------

try_end macro
    invoke SetUnhandledExceptionFilter, _previous_exception_filter_
endm

;==============================================================================
    .data
        _exception_return_address_  dd 0
        _previous_exception_filter_ dd 0
        exception_record            EXCEPTION_RECORD <>
    .code
;==============================================================================

PEXCEPTION_RECORD typedef ptr EXCEPTION_RECORD

EXCEPTION_NONCONTINUABLE_EXCEPTION  equ 0C0000025h
EXCEPTION_STACK_OVERFLOW            equ 0C00000FDh
EXCEPTION_INVALID_DISPOSITION       equ 0C0000026h
EXCEPTION_GUARD_PAGE                equ 080000001h
EXCEPTION_INVALID_HANDLE            equ 0C0000008h

ShowException proc uses ebx per:PEXCEPTION_RECORD
    mov ebx, per
    SWITCH [ebx].EXCEPTION_RECORD.ExceptionCode
        CASE EXCEPTION_ACCESS_VIOLATION
            printf("EXCEPTION_ACCESS_VIOLATION")
        CASE EXCEPTION_DATATYPE_MISALIGNMENT
            printf("EXCEPTION_DATATYPE_MISALIGNMENT")
        CASE EXCEPTION_BREAKPOINT
            printf("EXCEPTION_BREAKPOINT")
        CASE EXCEPTION_SINGLE_STEP
            printf("EXCEPTION_SINGLE_STEP")
        CASE EXCEPTION_ARRAY_BOUNDS_EXCEEDED
            printf("EXCEPTION_ARRAY_BOUNDS_EXCEEDED")
        CASE EXCEPTION_FLT_DENORMAL_OPERAND
            printf("EXCEPTION_FLT_DENORMAL_OPERAND")
        CASE EXCEPTION_FLT_DIVIDE_BY_ZERO
            printf("EXCEPTION_FLT_DIVIDE_BY_ZERO")
        CASE EXCEPTION_FLT_INEXACT_RESULT
            printf("EXCEPTION_FLT_INEXACT_RESULT")
        CASE EXCEPTION_FLT_INVALID_OPERATION
            printf("EXCEPTION_FLT_INVALID_OPERATION")
        CASE EXCEPTION_FLT_OVERFLOW
            printf("EXCEPTION_FLT_OVERFLOW")
        CASE EXCEPTION_FLT_STACK_CHECK
            printf("EXCEPTION_FLT_STACK_CHECK")
        CASE EXCEPTION_FLT_UNDERFLOW
            printf("EXCEPTION_FLT_UNDERFLOW")
        CASE EXCEPTION_INT_DIVIDE_BY_ZERO
            printf("EXCEPTION_INT_DIVIDE_BY_ZERO")
        CASE EXCEPTION_INT_OVERFLOW
            printf("EXCEPTION_INT_OVERFLOW")
        CASE EXCEPTION_PRIV_INSTRUCTION
            printf("EXCEPTION_PRIV_INSTRUCTION")
        CASE EXCEPTION_IN_PAGE_ERROR
            printf("EXCEPTION_IN_PAGE_ERROR")
        CASE EXCEPTION_ILLEGAL_INSTRUCTION
            printf("EXCEPTION_ILLEGAL_INSTRUCTION")
        CASE EXCEPTION_NONCONTINUABLE_EXCEPTION
            printf("EXCEPTION_NONCONTINUABLE_EXCEPTION")
        CASE EXCEPTION_STACK_OVERFLOW
            printf("EXCEPTION_STACK_OVERFLOW")
        CASE EXCEPTION_INVALID_DISPOSITION
            printf("EXCEPTION_INVALID_DISPOSITION")
        CASE EXCEPTION_GUARD_PAGE
            printf("EXCEPTION_GUARD_PAGE")
        CASE EXCEPTION_INVALID_HANDLE
            printf("EXCEPTION_INVALID_HANDLE")
        DEFAULT
            printf("exception code ")
            printf("%X",[ebx].EXCEPTION_RECORD.ExceptionCode)
    ENDSW
    printf(" @ %Xh\n",[ebx].EXCEPTION_RECORD.ExceptionAddress)
    ret
ShowException endp

;==============================================================================

;---------------------------------------------------
; This procedure is our top-level exception filter.
;---------------------------------------------------

try_filter proc lpexception_pointers:LPEXCEPTION_POINTERS

    ;-------------------------------------------------------------------------
    ; Copy the entire EXCEPTION_RECORD structure (80 bytes) to global memory.
    ;-------------------------------------------------------------------------

    mov edx, lpexception_pointers
    mov ecx, [edx].EXCEPTION_POINTERS.pExceptionRecord
    mov edx, offset exception_record
    DISP=0
    repeat (sizeof EXCEPTION_RECORD) / 4
        mov eax, [ecx+DISP]
        mov [edx+DISP], eax
        DISP=DISP+4
    endm

    ;------------------------------------------------------------------
    ; Modify the EIP member of the CONTEXT structure so execution will
    ; continue at the address passed to the try_begin macro, instead
    ; of continuing at the point of the exception (which would cause
    ; the exception to repeat).
    ;------------------------------------------------------------------

    mov edx, lpexception_pointers
    mov eax, [edx].EXCEPTION_POINTERS.ContextRecord
    push _exception_return_address_
    pop [eax].CONTEXT.regEip

    invoke ShowException, ADDR exception_record

    ;------------------------------------------------------------------------
    ; Returning EXCEPTION_CONTINUE_EXECUTION to the UnhandledExceptionFilter
    ; function will cause it to return and continue execution from the point
    ; specified in the EIP member of the CONTEXT structure.
    ;------------------------------------------------------------------------

    return EXCEPTION_CONTINUE_EXECUTION

try_filter endp

;==============================================================================
